/*
 *
 * UNICON - The Console Chinese & I18N
 * Copyright (c) 1999-2000
 *
 * This file is part of UNICON, a console Chinese & I18N
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * See the file COPYING directory of this archive
 * Author: see CREDITS
 */

#include <linux/sched.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/errno.h>
#include <linux/kd.h>
#include <linux/slab.h>
#include <linux/mm.h>
#include <linux/fb.h>
#include <linux/vt_kern.h>
#include <linux/version.h>
#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0))
#include <linux/console_struct.h>
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
#include "../console/fbcon.h"
#elif (LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0))
#include <video/fbcon.h>
#endif

#include "xl_hzfb.h"
#include <linux/fb_doublebyte.h>

extern signed char con2fb_map[MAX_NR_CONSOLES];
extern struct display fb_display[MAX_NR_CONSOLES];

int GetCurrentTTY (void)
{
   struct vc *q = &vc_cons [fg_console];
   struct vc_data *d = q->d;
   int unit = d->vc_num;
   return unit + 1;
}

struct fb_info *GetRgisteredFb (void)
{
   struct vc *q = &vc_cons [fg_console];
   struct vc_data *d = q->d;
   int unit = d->vc_num;
   return registered_fb[(int) con2fb_map[unit]];
}

int HzFbInit (HzFb_T *fbinfo)
{
   struct vc *q = &vc_cons [fg_console];
   struct vc_data *d = q->d;
   int unit = d->vc_num;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
   struct fb_info *info = registered_fb[(int) con2fb_map[unit]];
#else
   struct display *p = &fb_display [unit];
#endif

   if(!info)
   {
       printk("Please use vga=xxx as kernel's command\n");
       return 1;
   }
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
   fbinfo->width = info->var.xres;
   fbinfo->height = info->var.yres; 
   fbinfo->fb_mem = info->screen_base;
   fbinfo->mem_len = d->vc_screenbuf_size;
   fbinfo->bits_per_pixel = info->var.bits_per_pixel;
   fbinfo->line_length = info->fix.line_length;
#else
   fbinfo->width = p->var.xres;
   fbinfo->height = p->var.yres; 
   fbinfo->fb_mem = p->screen_base;
   fbinfo->mem_len = d->vc_screenbuf_size;
   fbinfo->bits_per_pixel = p->var.bits_per_pixel;
   fbinfo->line_length = p->line_length;
#endif

   return 0;
}

int HzFbExit (HzFb_T *fbinfo)
{
    return 0;
}

void FbPutAscii (HzFb_T *fbinfo, int x0, int y0, long color, unsigned char ch)
{
    struct vc *q = &vc_cons [fg_console];
    struct vc_data *conp = q->d;
    int c, ypos, xpos;

    c = ch | (color & 0xff) << 8; 
    xpos = x0; 
    ypos = y0;

    fbcon_putc(conp, c, ypos, xpos);
}
 
void FbPutChinese (HzFb_T *fbinfo, int x0, int y0, 
	           long color, unsigned char ch, unsigned char cl)
{
    struct vc *q = &vc_cons [fg_console];
    struct vc_data *conp = q->d;
    int c, ypos, xpos;

    c = ch | ((color & 0xff) << 8)| ((cl+DB_LEFT) << 16) ;
    xpos = x0;
    ypos = y0;

    fbcon_putc(conp, c, ypos, xpos);

    xpos ++;
    c = cl | ((color & 0xff) << 8)| ((ch+DB_RIGHT) << 16) ;
    fbcon_putc(conp, c, ypos, xpos);
}

void FbClearRect (HzFb_T *fbinfo, unsigned char cl, int y1, int y2)
{
   int i;
   for (i = 0; i < fbinfo->width/8; i++) 
       FbPutAscii (fbinfo, i, y1, cl, ' ');
}

int GetFbHeight (HzFb_T *fbinfo)
{
    return fbinfo->height; 
}

int GetFbWidth (HzFb_T *fbinfo)
{
    return fbinfo->width; 
}

